@@ -1,8 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-
 import logging
 from rq import get_current_connection
 from rq import Worker
