@@ -1,9 +1,6 @@
-# -*- coding: utf-8 -*-
 """
 Some dummy tasks that are well-suited for generating load for testing purposes.
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 import random
 import time
