@@ -1,7 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import errno
 import logging
 import os
@@ -22,7 +18,7 @@ try:
     from signal import SIGKILL
 except ImportError:
     from signal import SIGTERM as SIGKILL
-
+from contextlib import suppress
 import redis.exceptions
 
 from . import worker_registration
@@ -40,7 +36,8 @@ from .queue import Queue
 from .registry import FailedJobRegistry, StartedJobRegistry, clean_registries
 from .scheduler import RQScheduler
 from .suspension import is_suspended
-from .timeouts import JobTimeoutException, HorseMonitorTimeoutException, UnixSignalDeathPenalty
+from .timeouts import (JobTimeoutException, HorseMonitorTimeoutException,
+                       UnixSignalDeathPenalty, TimerDeathPenalty)
 from .utils import (backend_class, ensure_list, get_version,
                     make_colorizer, utcformat, utcnow, utcparse)
 from .version import VERSION
@@ -57,7 +54,6 @@ green = make_colorizer('darkgreen')
 yellow = make_colorizer('darkyellow')
 blue = make_colorizer('darkblue')
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -229,7 +225,11 @@ class Worker:
                 )
                 self.ip_address = 'unknown'
             else:
-                self.ip_address = [client['addr'] for client in connection.client_list() if client['name'] == self.name][0]
+                self.ip_address = [
+                    client['addr']
+                    for client in connection.client_list()
+                    if client['name'] == self.name
+                ][0]
         else:
             self.hostname = None
             self.pid = None
@@ -922,7 +922,6 @@ class Worker:
         """
 
         with self.connection.pipeline() as pipeline:
-            self.set_state(WorkerStatus.BUSY, pipeline=pipeline)
             self.set_current_job_id(job.id, pipeline=pipeline)
             self.set_current_job_working_time(0, pipeline=pipeline)
 
@@ -975,6 +974,8 @@ class Worker:
                                                         job_class=self.job_class, serializer=job.serializer)
                 failed_job_registry.add(job, ttl=job.failure_ttl,
                                         exc_string=exc_string, pipeline=pipeline)
+                with suppress(redis.exceptions.ConnectionError):
+                    pipeline.execute()
 
             self.set_current_job_id(None, pipeline=pipeline)
             self.increment_failed_job_count(pipeline)
@@ -985,9 +986,14 @@ class Worker:
 
             if retry:
                 job.retry(queue, pipeline)
+                enqueue_dependents = False
+            else:
+                enqueue_dependents = True
 
             try:
                 pipeline.execute()
+                if enqueue_dependents:
+                    queue.enqueue_dependents(job)
             except Exception:
                 # Ensure that custom exception handlers are called
                 # even if Redis is down
@@ -995,6 +1001,7 @@ class Worker:
 
     def handle_job_success(self, job, queue, started_job_registry):
         self.log.debug('Handling successful execution of job %s', job.id)
+
         with self.connection.pipeline() as pipeline:
             while True:
                 try:
@@ -1192,10 +1199,11 @@ class Worker:
 
 
 class SimpleWorker(Worker):
-
     def execute_job(self, job, queue):
         """Execute job in same thread/process, do not fork()"""
-        return self.perform_job(job, queue)
+        self.set_state(WorkerStatus.BUSY)
+        self.perform_job(job, queue)
+        self.set_state(WorkerStatus.IDLE)
 
     def get_heartbeat_ttl(self, job):
         # "-1" means that jobs never timeout. In this case, we should _not_ do -1 + 60 = 59.
@@ -1255,6 +1263,7 @@ class RoundRobinWorker(Worker):
     """
     Modified version of Worker that dequeues jobs from the queues using a round-robin strategy.
     """
+
     def reorder_queues(self, reference_queue):
         pos = self._ordered_queues.index(reference_queue)
         self._ordered_queues = self._ordered_queues[pos + 1:] + self._ordered_queues[:pos + 1]
